package com.xyzc.start.controller;

import com.xyzc.common.core.utils.Result;
import com.xyzc.start.domain.DrawAssistant;
import com.xyzc.start.domain.query.AppQuery;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.service.IDiscoverService;
import com.xyzc.start.service.IDrawAssistantService;
import jakarta.validation.Valid;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.List;


/**
 * 发现
 *
 * @author 杨兴卓
 */
@RequiredArgsConstructor
@RestController
@RequestMapping("/discover")
@CrossOrigin
public class DiscoverPageController {

    private final IDiscoverService discoverService;

    private final IDrawAssistantService drawAssistantService;

    /**
     * 绘画广场
     */
    @GetMapping("/drawSquare")
    public Result<List<DrawGraphicsSquareVo>> drawSquare(Long graphicsId, String keyword) {
        List<DrawGraphicsSquareVo> drawGraphicsList = discoverService.queryGraphics(graphicsId, keyword);

        return Result.ok(drawGraphicsList);
    }


    /**
     * 绘画模型
     */
    @GetMapping("/drawModel")
    public Result<List<DrawModelCollectVo>> drawModel(Long drawModelId, String keyword) {
        List<DrawModelCollectVo> drawModelList = discoverService.queryDrawModelList(drawModelId, keyword);

        return Result.ok(drawModelList);
    }

    /**
     * 分页查询反向提示词
     */
    @GetMapping("/reversePrompt")
    public Result<List<DrawReversePromptVo>> reversePrompt(Long reversePromptId, String keyword, Integer length) {
        List<DrawReversePromptVo> reversePromptList = discoverService.queryReversePromptList(reversePromptId, keyword, length);

        return Result.ok(reversePromptList);
    }

    /**
     * 查询所有的绘画助手
     */
    @GetMapping("/drawAssistantList")
    public Result<List<DrawAssistantVo>> drawAssistantList() {
        List<DrawAssistantVo> drawAssistantList = drawAssistantService.queryDrawAssistantList();

        return Result.ok(drawAssistantList);
    }

    /**
     * 查询具体绘画助手的提示词
     */
    @GetMapping("/drawAssistant/{assistantId}")
    public Result<List<DrawPromptConfigVo>> drawAssistant(@PathVariable Integer assistantId, Long promptId, String keyword, Integer length) {
        List<DrawPromptConfigVo> drawPromptConfigList = drawAssistantService.queryDrawPromptConfigList(assistantId, promptId, keyword, length);

        return Result.ok(drawPromptConfigList);
    }

    /**
     * 进入具体绘画模型
     */
    @GetMapping("/drawModelDetail/{modelId}")
    public Result<List<Void>> queryDrawModel(@PathVariable String modelId) {

        return Result.ok();
    }

    /**
     * 应用精选、特色、玩转
     *
     */
    @GetMapping("/selectedApp")
    public Result<List<BaseUserSelectAppVo>> selectedApp(@Valid AppQuery query) {
        List<BaseUserSelectAppVo> appPageList = discoverService.queryAppPageList(query);

        return Result.ok(appPageList);
    }


    /**
     * 数字人列表
     *
     */
    @GetMapping("/digitalHuman")
    public Result<List<Void>> digitalHuman() {

        return Result.ok();
    }

}
