package com.xyzc.start.service.impl;

import com.xyzc.common.core.exception.ErrorCode;
import com.xyzc.common.core.exception.ServerException;
import com.xyzc.common.core.utils.AssertUtils;
import com.xyzc.common.core.utils.MapstructUtils;
import com.xyzc.common.web.security.user.SecurityUser;
import com.xyzc.start.constant.BaseUserConst;
import com.xyzc.start.domain.*;
import com.xyzc.start.domain.bo.AgentCardSecretBo;
import com.xyzc.start.domain.bo.BaseUserCollectBo;
import com.xyzc.start.domain.bo.HobbyBo;
import com.xyzc.start.domain.vo.*;
import com.xyzc.start.mapper.*;
import com.xyzc.start.service.IUserInfoService;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.List;


@Service
@RequiredArgsConstructor
public class UserInfoServiceImpl implements IUserInfoService {

    private final BaseUserMapper baseUserMapper;
    private final IndustryMapper industryMapper;
    private final BaseUserPhotoFolderMapper photoFolderMapper;

    private final FuelFlowMapper fuelFlowMapper;

    private final AppMapper appMapper;
    private final BaseUserCollectMapper baseUserCollectMapper;

    private final DrawGraphicsMapper drawGraphicsMapper;
    private final DrawModelMapper drawModelMapper;
    private final BaseUserMemberCardMapper baseUserMemberCardMapper;
    private final MemberCardMapper memberCardMapper;

    /**
     * 获取当前用户信息
     *
     * @return
     */
    @Override
    public BaseUserVo queryCurrent() {
        BaseUserVo baseUser = baseUserMapper.queryCurrentInfo(SecurityUser.getUserId());
        // todo 后期加入次数

        return baseUser;
    }

    /**
     * 保存用户爱好
     *
     * @param hobbyBo HobbyBo
     */
    @Override
    public void saveHobby(HobbyBo hobbyBo) {

        baseUserMapper.saveHobbys(SecurityUser.getUserId(), hobbyBo);
    }


    /**
     * 获取用户信息
     *
     * @return
     */
    @Override
    public BaseUserVo queryInfo() {
        BaseUserVo baseUserVo = baseUserMapper.queryBaseUserVoById(SecurityUser.getUserId());

        // TODO： 后续计算
        baseUserVo.setUseCount(108);

        // 添加用户拥有的会员卡
        List<BaseUserMemberCardVo> memberCardVos = baseUserMapper.queryMyMemberCardByUserId(baseUserVo.getId());
        baseUserVo.setMemberCardVoList(memberCardVos);

        // 添加用户会员卡数量
        baseUserVo.setMemberCardCount(memberCardVos.size());


        return baseUserVo;
    }

    /**
     * 获取所有的行业
     *
     * @return
     */
    @Override
    public List<IndustryVo> getIndustryAll() {

        return industryMapper.selectVoList(null);
    }

    /**
     * 根据 id获取用户图画
     *
     * @param id Long
     * @return BaseUserPhotoFolderVo
     */
    @Override
    public BaseUserPhotoFolderVo queryPhotoById(Long id) {

        return photoFolderMapper.queryPhotoFolderById(SecurityUser.getUserId(), id);
    }

    /**
     * 获取用户画夹
     *
     * @param id
     * @return
     */
    @Override
    public List<BaseUserPhotoFolderVo> photoFolder(Long id) {

        return photoFolderMapper.queryPhotoFolder(SecurityUser.getUserId(), id);
    }

    /**
     * @param id
     */
    @Override
    public void deletePhoto(Long id) {
        //TODO 记得后期删除图片
        Boolean flag = photoFolderMapper.deletePhotoById(SecurityUser.getUserId(), id);
        if (!flag) {
            throw new ServerException(ErrorCode.USER_DELETE_IMAGE_ERROR);
        }
    }

    /**
     * @param ids
     */
    @Override
    public void deletePhotos(List<Long> ids) {
//        photoFolderMapper.deletePhotosByIds(SecurityUser.getUserId(), ids);
    }

    /**
     * 获取用户燃料流水
     */
    @Override
    public List<BaseUserFuelFlowVo> queryFuelFlow(Long flowId) {

        return fuelFlowMapper.queryFuelFlowList(SecurityUser.getUserId(), flowId);
    }

    /**
     * 销卡
     *
     * @param exchangeBo
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void exchange(AgentCardSecretBo exchangeBo) {
        // TODO： 注意，这里要限制用户每天最多兑换10次，输入10次锁住不让再兑换，并且记录重要事件日志

        //销卡的步骤
        // 1. 查询该卡密是否有效，如果无效，则抛出异常


        // 2. 查询该卡密是否使用过，如果使用过，则抛出异常

        // 3. 查询该卡密是否属于该用户所属的代理商，如果不属于，则抛出异常

        // 4. 开始核销卡密，先将其状态修改为 正在核销

    }

    /**
     * 查询会员卡使用详情
     *
     * @param cardId
     * @return
     */
    @Override
    public BaseUserMemberCardVo queryCardUseDetail(Long cardId) {
        // todo: 目前商户是写死的
        BaseUserMemberCardVo baseUserMemberCardVo = baseUserMemberCardMapper.queryMemberCard(SecurityUser.getUserId(), cardId, 1810310843224322049L);

        List<MemberCardUseDetail> objects = new ArrayList<>();

        MemberCardUseDetail memberCardUseDetail = new MemberCardUseDetail();
        memberCardUseDetail.setModelId(1242112L);
        memberCardUseDetail.setUseCount(32);
        memberCardUseDetail.setUnit(0);
        memberCardUseDetail.setModelName("GPT 3.5");
        memberCardUseDetail.setAlreadyUseCount(123);

        MemberCardUseDetail memberCardUseDetail2 = new MemberCardUseDetail();
        memberCardUseDetail2.setModelId(12421122L);
        memberCardUseDetail2.setUseCount(32);
        memberCardUseDetail2.setUnit(1);
        memberCardUseDetail2.setModelName("GPT 4.o");
        memberCardUseDetail2.setAlreadyUseCount(1323);

        MemberCardUseDetail memberCardUseDetail3 = new MemberCardUseDetail();
        memberCardUseDetail3.setModelId(124242112L);
        memberCardUseDetail3.setUseCount(3432);
        memberCardUseDetail3.setModelName("SD 2.2");
        memberCardUseDetail3.setAlreadyUseCount(1223);
        memberCardUseDetail3.setUnit(3);

        MemberCardUseDetail memberCardUseDetail4 = new MemberCardUseDetail();
        memberCardUseDetail4.setModelId(1242112L);
        memberCardUseDetail4.setUseCount(32);
        memberCardUseDetail4.setModelName("GPT 3.5");
        memberCardUseDetail4.setUnit(1);
        memberCardUseDetail4.setAlreadyUseCount(1243);

        MemberCardUseDetail memberCardUseDetail5 = new MemberCardUseDetail();
        memberCardUseDetail5.setModelId(1242112L);
        memberCardUseDetail5.setUseCount(32);
        memberCardUseDetail5.setModelName("GPT 3.5");
        memberCardUseDetail5.setUnit(2);
        memberCardUseDetail5.setAlreadyUseCount(1273);


        MemberCardUseDetail memberCardUseDetail7 = new MemberCardUseDetail();
        memberCardUseDetail7.setModelId(1242112L);
        memberCardUseDetail7.setUseCount(32);
        memberCardUseDetail7.setModelName("GPT 3.5");
        memberCardUseDetail7.setAlreadyUseCount(1283);
        memberCardUseDetail7.setUnit(3);


        MemberCardUseDetail memberCardUseDetail8 = new MemberCardUseDetail();
        memberCardUseDetail8.setModelId(1242112L);
        memberCardUseDetail8.setUseCount(32);
        memberCardUseDetail8.setUnit(3);
        memberCardUseDetail8.setModelName("GPT 3.5");
        memberCardUseDetail8.setAlreadyUseCount(12833);

        MemberCardUseDetail memberCardUseDetail9 = new MemberCardUseDetail();
        memberCardUseDetail9.setModelId(1242112L);
        memberCardUseDetail9.setUseCount(32);
        memberCardUseDetail9.setUnit(2);
        memberCardUseDetail9.setModelName("火绒 1.5");
        memberCardUseDetail9.setAlreadyUseCount(123);

        MemberCardUseDetail memberCardUseDetail10 = new MemberCardUseDetail();
        memberCardUseDetail10.setModelId(124L);
        memberCardUseDetail10.setUseCount(32);
        memberCardUseDetail10.setUnit(1);
        memberCardUseDetail10.setModelName("火山 3.5");
        memberCardUseDetail10.setAlreadyUseCount(1203);


        objects.add(memberCardUseDetail);
        objects.add(memberCardUseDetail2);
        objects.add(memberCardUseDetail3);
        objects.add(memberCardUseDetail4);
        objects.add(memberCardUseDetail5);
        objects.add(memberCardUseDetail7);
        objects.add(memberCardUseDetail8);
        objects.add(memberCardUseDetail9);
        objects.add(memberCardUseDetail10);

        baseUserMemberCardVo.setMemberCardUseDetail(objects);
        return baseUserMemberCardVo;
    }


    /**
     * 收藏或取消收藏
     *
     * @param collectBo 收藏对象
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void collectOrCancel(BaseUserCollectBo collectBo) {
        BaseUserCollect baseUserCollect = baseUserCollectMapper.queryCollectByIdAndCollectId(SecurityUser.getUserId(), collectBo.getCollectId(), collectBo.getType());
        if (baseUserCollect == null) {
            Object obj = null;
            // 查询收藏对象是否存在，不存在抛出异常
            if (BaseUserConst.COLLECT_DRAW_MODEL.equals(collectBo.getType())) {
                obj = (Object) drawModelMapper.selectById(collectBo.getCollectId());
            } else if (BaseUserConst.COLLECT_IMAGE.equals(collectBo.getType())) {
                obj = (Object) drawGraphicsMapper.selectById(collectBo.getCollectId());
            } else if (BaseUserConst.COLLECT_APP.equals(collectBo.getType())) {
                obj = (Object) appMapper.selectById(collectBo.getCollectId());
            }
            AssertUtils.notNull(obj, ErrorCode.COLLECT_NOT_EXIST);

            // 转换用户收藏
            BaseUserCollect convert = MapstructUtils.convert(collectBo, BaseUserCollect.class);
            convert.setBaseUserId(SecurityUser.getUserId());

            baseUserCollectMapper.insert(convert);
            // 更新对应的收藏量
            if (BaseUserConst.COLLECT_DRAW_MODEL.equals(collectBo.getType()) &&
                    BaseUserConst.IS_COLLECT.equals(collectBo.getType())) {
                DrawModel drawModel = (DrawModel) obj;
                drawModel.setUseCount(drawModel.getUseCount() + 1);
                drawModelMapper.updateById(drawModel);
            } else if (BaseUserConst.COLLECT_IMAGE.equals(collectBo.getType()) &&
                    BaseUserConst.IS_COLLECT.equals(collectBo.getType())) {
                DrawGraphics graphics = (DrawGraphics) obj;
                graphics.setUseCount(graphics.getUseCount() + 1);
                drawGraphicsMapper.updateById(graphics);
            } else if (BaseUserConst.COLLECT_APP.equals(collectBo.getType())) {
                App app = (App) obj;
                app.setUseCount(app.getUseCount() + 1);
                appMapper.updateById(app);
            }

            return;
        }

        // 删除用户收藏
        baseUserCollectMapper.deleteById(baseUserCollect.getId());

        // 插入用户收藏
        baseUserCollect.setId(null);
        baseUserCollect.setIsCollect(collectBo.getIsCollect());


        baseUserCollectMapper.insert(baseUserCollect);
    }

    /**
     * 根据应用id扣除对应的燃料
     *
     * @param appId
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deduct(Long appId, Long aiModelId) {
        // 首先获取个人燃料剩余量
        BaseUser baseUser = baseUserMapper.selectVoByIdForUpdate(SecurityUser.getUserId());

        // 然后获取应用对应的ai模型燃料扣除量
        AiModelVo aiModelVo = appMapper.queryAiModelFuel(appId, aiModelId);

        // 未查到应用报错
        AssertUtils.notNull(aiModelVo, ErrorCode.AI_MODEL_NO_EXIST_CODE);
        baseUser.setSurplusFuel(baseUser.getSurplusFuel() - aiModelVo.getConsumeFuelNumber());

        // 判断扣除后的用户剩余燃料是否小于0，如果小于0，则抛出异常
        AssertUtils.surplusFuel(baseUser.getSurplusFuel());

        // 然后扣除燃料返回成功
        baseUserMapper.updateById(baseUser);
    }
}

